# Generated from ObjectiveCPreprocessorParser.g4 by ANTLR 4.7.1
# encoding: utf-8
from __future__ import print_function
from QBCodeSpecs.antlr4 import *
from io import StringIO
import sys

def serializedATN():
    with StringIO() as buf:
        buf.write(u"\3\u608b\ua72a\u8133\ub9ed\u417c\u3be7\u7786\u5964\3")
        buf.write(u"\u00e0a\4\2\t\2\4\3\t\3\4\4\t\4\3\2\3\2\3\2\3\2\3\2\3")
        buf.write(u"\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3")
        buf.write(u"\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3")
        buf.write(u"\2\3\2\3\2\3\2\5\2,\n\2\5\2.\n\2\3\3\6\3\61\n\3\r\3\16")
        buf.write(u"\3\62\3\4\3\4\3\4\3\4\3\4\3\4\3\4\3\4\3\4\3\4\5\4?\n")
        buf.write(u"\4\3\4\3\4\3\4\3\4\3\4\3\4\3\4\3\4\3\4\3\4\3\4\5\4L\n")
        buf.write(u"\4\5\4N\n\4\3\4\3\4\3\4\3\4\3\4\3\4\3\4\3\4\3\4\3\4\3")
        buf.write(u"\4\3\4\7\4\\\n\4\f\4\16\4_\13\4\3\4\2\3\6\5\2\4\6\2\6")
        buf.write(u"\3\2\u00bc\u00bd\3\2\u00df\u00e0\3\2\u00cf\u00d0\3\2")
        buf.write(u"\u00d3\u00d6\2w\2-\3\2\2\2\4\60\3\2\2\2\6M\3\2\2\2\b")
        buf.write(u"\t\7\u00b8\2\2\t\n\t\2\2\2\n.\5\4\3\2\13\f\7\u00b8\2")
        buf.write(u"\2\f\r\7\u00c1\2\2\r.\5\6\4\2\16\17\7\u00b8\2\2\17\20")
        buf.write(u"\7\u00c2\2\2\20.\5\6\4\2\21\22\7\u00b8\2\2\22.\7\u00c3")
        buf.write(u"\2\2\23\24\7\u00b8\2\2\24.\7\u00c7\2\2\25\26\7\u00b8")
        buf.write(u"\2\2\26\27\7\u00c5\2\2\27.\7\u00d8\2\2\30\31\7\u00b8")
        buf.write(u"\2\2\31\32\7\u00c6\2\2\32.\7\u00d8\2\2\33\34\7\u00b8")
        buf.write(u"\2\2\34\35\7\u00c4\2\2\35.\7\u00d8\2\2\36\37\7\u00b8")
        buf.write(u"\2\2\37 \7\u00be\2\2 .\5\4\3\2!\"\7\u00b8\2\2\"#\7\u00ca")
        buf.write(u"\2\2#.\5\4\3\2$%\7\u00b8\2\2%&\7\u00cb\2\2&.\5\4\3\2")
        buf.write(u"\'(\7\u00b8\2\2()\7\u00bf\2\2)+\7\u00d8\2\2*,\5\4\3\2")
        buf.write(u"+*\3\2\2\2+,\3\2\2\2,.\3\2\2\2-\b\3\2\2\2-\13\3\2\2\2")
        buf.write(u"-\16\3\2\2\2-\21\3\2\2\2-\23\3\2\2\2-\25\3\2\2\2-\30")
        buf.write(u"\3\2\2\2-\33\3\2\2\2-\36\3\2\2\2-!\3\2\2\2-$\3\2\2\2")
        buf.write(u"-\'\3\2\2\2.\3\3\2\2\2/\61\t\3\2\2\60/\3\2\2\2\61\62")
        buf.write(u"\3\2\2\2\62\60\3\2\2\2\62\63\3\2\2\2\63\5\3\2\2\2\64")
        buf.write(u"\65\b\4\1\2\65N\7\u00c8\2\2\66N\7\u00c9\2\2\67N\7\u00d9")
        buf.write(u"\2\28N\7\u00d7\2\29>\7\u00d8\2\2:;\7\u00cd\2\2;<\5\6")
        buf.write(u"\4\2<=\7\u00ce\2\2=?\3\2\2\2>:\3\2\2\2>?\3\2\2\2?N\3")
        buf.write(u"\2\2\2@A\7\u00cd\2\2AB\5\6\4\2BC\7\u00ce\2\2CN\3\2\2")
        buf.write(u"\2DE\7\u00cc\2\2EN\5\6\4\bFK\7\u00c0\2\2GL\7\u00d8\2")
        buf.write(u"\2HI\7\u00cd\2\2IJ\7\u00d8\2\2JL\7\u00ce\2\2KG\3\2\2")
        buf.write(u"\2KH\3\2\2\2LN\3\2\2\2M\64\3\2\2\2M\66\3\2\2\2M\67\3")
        buf.write(u"\2\2\2M8\3\2\2\2M9\3\2\2\2M@\3\2\2\2MD\3\2\2\2MF\3\2")
        buf.write(u"\2\2N]\3\2\2\2OP\f\7\2\2PQ\t\4\2\2Q\\\5\6\4\bRS\f\6\2")
        buf.write(u"\2ST\7\u00d1\2\2T\\\5\6\4\7UV\f\5\2\2VW\7\u00d2\2\2W")
        buf.write(u"\\\5\6\4\6XY\f\4\2\2YZ\t\5\2\2Z\\\5\6\4\5[O\3\2\2\2[")
        buf.write(u"R\3\2\2\2[U\3\2\2\2[X\3\2\2\2\\_\3\2\2\2][\3\2\2\2]^")
        buf.write(u"\3\2\2\2^\7\3\2\2\2_]\3\2\2\2\n+-\62>KM[]")
        return buf.getvalue()


class ObjectiveCPreprocessorParser ( Parser ):

    grammarFileName = "ObjectiveCPreprocessorParser.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ u"<INVALID>", u"'auto'", u"'break'", u"'case'", u"'char'", 
                     u"'const'", u"'continue'", u"'default'", u"'do'", u"'double'", 
                     u"<INVALID>", u"'enum'", u"'extern'", u"'float'", u"'for'", 
                     u"'goto'", u"<INVALID>", u"'inline'", u"'int'", u"'long'", 
                     u"'register'", u"'restrict'", u"'return'", u"'short'", 
                     u"'signed'", u"'sizeof'", u"'static'", u"'struct'", 
                     u"'switch'", u"'typedef'", u"'union'", u"'unsigned'", 
                     u"'void'", u"'volatile'", u"'while'", u"'_Bool'", u"'_Complex'", 
                     u"'_Imaginery'", u"'true'", u"'false'", u"'BOOL'", 
                     u"'Class'", u"'bycopy'", u"'byref'", u"'id'", u"'IMP'", 
                     u"'in'", u"'inout'", u"'nil'", u"'NO'", u"'NULL'", 
                     u"'oneway'", u"'out'", u"'Protocol'", u"'SEL'", u"'self'", 
                     u"'super'", u"'YES'", u"'@autoreleasepool'", u"'@catch'", 
                     u"'@class'", u"'@dynamic'", u"'@encode'", u"'@end'", 
                     u"'@finally'", u"'@implementation'", u"'@interface'", 
                     u"'@import'", u"'@package'", u"'@protocol'", u"'@optional'", 
                     u"'@private'", u"'@property'", u"'@protected'", u"'@public'", 
                     u"'@required'", u"'@selector'", u"'@synchronized'", 
                     u"'@synthesize'", u"'@throw'", u"'@try'", u"'atomic'", 
                     u"'nonatomic'", u"'retain'", u"'__attribute__'", u"'__autoreleasing'", 
                     u"'__block'", u"'__bridge'", u"'__bridge_retained'", 
                     u"'__bridge_transfer'", u"'__covariant'", u"'__contravariant'", 
                     u"'__deprecated'", u"'__kindof'", u"'__strong'", u"<INVALID>", 
                     u"'__unsafe_unretained'", u"'__unused'", u"'__weak'", 
                     u"<INVALID>", u"<INVALID>", u"<INVALID>", u"'null_resettable'", 
                     u"'NS_INLINE'", u"'NS_ENUM'", u"'NS_OPTIONS'", u"'assign'", 
                     u"'copy'", u"'getter'", u"'setter'", u"'strong'", u"'readonly'", 
                     u"'readwrite'", u"'weak'", u"'unsafe_unretained'", 
                     u"'IBOutlet'", u"'IBOutletCollection'", u"'IBInspectable'", 
                     u"'IB_DESIGNABLE'", u"<INVALID>", u"<INVALID>", u"<INVALID>", 
                     u"<INVALID>", u"<INVALID>", u"'__TVOS_PROHIBITED'", 
                     u"<INVALID>", u"<INVALID>", u"<INVALID>", u"'{'", u"'}'", 
                     u"'['", u"']'", u"';'", u"','", u"'.'", u"'->'", u"'@'", 
                     u"'='", u"<INVALID>", u"<INVALID>", u"<INVALID>", u"'~'", 
                     u"'?'", u"':'", u"<INVALID>", u"<INVALID>", u"<INVALID>", 
                     u"<INVALID>", u"<INVALID>", u"<INVALID>", u"'++'", 
                     u"'--'", u"'+'", u"'-'", u"'*'", u"'/'", u"'&'", u"'|'", 
                     u"'^'", u"'%'", u"'+='", u"'-='", u"'*='", u"'/='", 
                     u"'&='", u"'|='", u"'^='", u"'%='", u"'<<='", u"'>>='", 
                     u"'...'", u"<INVALID>", u"<INVALID>", u"<INVALID>", 
                     u"<INVALID>", u"<INVALID>", u"<INVALID>", u"<INVALID>", 
                     u"<INVALID>", u"<INVALID>", u"<INVALID>", u"'\\'", 
                     u"<INVALID>", u"<INVALID>", u"<INVALID>", u"<INVALID>", 
                     u"<INVALID>", u"<INVALID>", u"<INVALID>", u"<INVALID>", 
                     u"'defined'", u"<INVALID>", u"'elif'", u"<INVALID>", 
                     u"'undef'", u"'ifdef'", u"'ifndef'", u"'endif'" ]

    symbolicNames = [ u"<INVALID>", u"AUTO", u"BREAK", u"CASE", u"CHAR", 
                      u"CONST", u"CONTINUE", u"DEFAULT", u"DO", u"DOUBLE", 
                      u"ELSE", u"ENUM", u"EXTERN", u"FLOAT", u"FOR", u"GOTO", 
                      u"IF", u"INLINE", u"INT", u"LONG", u"REGISTER", u"RESTRICT", 
                      u"RETURN", u"SHORT", u"SIGNED", u"SIZEOF", u"STATIC", 
                      u"STRUCT", u"SWITCH", u"TYPEDEF", u"UNION", u"UNSIGNED", 
                      u"VOID", u"VOLATILE", u"WHILE", u"BOOL_", u"COMPLEX", 
                      u"IMAGINERY", u"TRUE", u"FALSE", u"BOOL", u"Class", 
                      u"BYCOPY", u"BYREF", u"ID", u"IMP", u"IN", u"INOUT", 
                      u"NIL", u"NO", u"NULL", u"ONEWAY", u"OUT", u"PROTOCOL_", 
                      u"SEL", u"SELF", u"SUPER", u"YES", u"AUTORELEASEPOOL", 
                      u"CATCH", u"CLASS", u"DYNAMIC", u"ENCODE", u"END", 
                      u"FINALLY", u"IMPLEMENTATION", u"INTERFACE", u"IMPORT", 
                      u"PACKAGE", u"PROTOCOL", u"OPTIONAL", u"PRIVATE", 
                      u"PROPERTY", u"PROTECTED", u"PUBLIC", u"REQUIRED", 
                      u"SELECTOR", u"SYNCHRONIZED", u"SYNTHESIZE", u"THROW", 
                      u"TRY", u"ATOMIC", u"NONATOMIC", u"RETAIN", u"ATTRIBUTE", 
                      u"AUTORELEASING_QUALIFIER", u"BLOCK", u"BRIDGE", u"BRIDGE_RETAINED", 
                      u"BRIDGE_TRANSFER", u"COVARIANT", u"CONTRAVARIANT", 
                      u"DEPRECATED", u"KINDOF", u"STRONG_QUALIFIER", u"TYPEOF", 
                      u"UNSAFE_UNRETAINED_QUALIFIER", u"UNUSED", u"WEAK_QUALIFIER", 
                      u"NULL_UNSPECIFIED", u"NULLABLE", u"NONNULL", u"NULL_RESETTABLE", 
                      u"NS_INLINE", u"NS_ENUM", u"NS_OPTIONS", u"ASSIGN", 
                      u"COPY", u"GETTER", u"SETTER", u"STRONG", u"READONLY", 
                      u"READWRITE", u"WEAK", u"UNSAFE_UNRETAINED", u"IB_OUTLET", 
                      u"IB_OUTLET_COLLECTION", u"IB_INSPECTABLE", u"IB_DESIGNABLE", 
                      u"NS_ASSUME_NONNULL_BEGIN", u"NS_ASSUME_NONNULL_END", 
                      u"EXTERN_SUFFIX", u"IOS_SUFFIX", u"MAC_SUFFIX", u"TVOS_PROHIBITED", 
                      u"IDENTIFIER", u"LP", u"RP", u"LBRACE", u"RBRACE", 
                      u"LBRACK", u"RBRACK", u"SEMI", u"COMMA", u"DOT", u"STRUCTACCESS", 
                      u"AT", u"ASSIGNMENT", u"GT", u"LT", u"BANG", u"TILDE", 
                      u"QUESTION", u"COLON", u"EQUAL", u"LE", u"GE", u"NOTEQUAL", 
                      u"AND", u"OR", u"INC", u"DEC", u"ADD", u"SUB", u"MUL", 
                      u"DIV", u"BITAND", u"BITOR", u"BITXOR", u"MOD", u"ADD_ASSIGN", 
                      u"SUB_ASSIGN", u"MUL_ASSIGN", u"DIV_ASSIGN", u"AND_ASSIGN", 
                      u"OR_ASSIGN", u"XOR_ASSIGN", u"MOD_ASSIGN", u"LSHIFT_ASSIGN", 
                      u"RSHIFT_ASSIGN", u"ELIPSIS", u"CHARACTER_LITERAL", 
                      u"STRING_START", u"HEX_LITERAL", u"OCTAL_LITERAL", 
                      u"BINARY_LITERAL", u"DECIMAL_LITERAL", u"FLOATING_POINT_LITERAL", 
                      u"WS", u"MULTI_COMMENT", u"SINGLE_COMMENT", u"BACKSLASH", 
                      u"SHARP", u"STRING_NEWLINE", u"STRING_END", u"STRING_VALUE", 
                      u"DIRECTIVE_IMPORT", u"DIRECTIVE_INCLUDE", u"DIRECTIVE_PRAGMA", 
                      u"DIRECTIVE_DEFINE", u"DIRECTIVE_DEFINED", u"DIRECTIVE_IF", 
                      u"DIRECTIVE_ELIF", u"DIRECTIVE_ELSE", u"DIRECTIVE_UNDEF", 
                      u"DIRECTIVE_IFDEF", u"DIRECTIVE_IFNDEF", u"DIRECTIVE_ENDIF", 
                      u"DIRECTIVE_TRUE", u"DIRECTIVE_FALSE", u"DIRECTIVE_ERROR", 
                      u"DIRECTIVE_WARNING", u"DIRECTIVE_BANG", u"DIRECTIVE_LP", 
                      u"DIRECTIVE_RP", u"DIRECTIVE_EQUAL", u"DIRECTIVE_NOTEQUAL", 
                      u"DIRECTIVE_AND", u"DIRECTIVE_OR", u"DIRECTIVE_LT", 
                      u"DIRECTIVE_GT", u"DIRECTIVE_LE", u"DIRECTIVE_GE", 
                      u"DIRECTIVE_STRING", u"DIRECTIVE_ID", u"DIRECTIVE_DECIMAL_LITERAL", 
                      u"DIRECTIVE_FLOAT", u"DIRECTIVE_NEWLINE", u"DIRECTIVE_MULTI_COMMENT", 
                      u"DIRECTIVE_SINGLE_COMMENT", u"DIRECTIVE_BACKSLASH_NEWLINE", 
                      u"DIRECTIVE_TEXT_NEWLINE", u"DIRECTIVE_TEXT" ]

    RULE_directive = 0
    RULE_directiveText = 1
    RULE_preprocessorExpression = 2

    ruleNames =  [ u"directive", u"directiveText", u"preprocessorExpression" ]

    EOF = Token.EOF
    AUTO=1
    BREAK=2
    CASE=3
    CHAR=4
    CONST=5
    CONTINUE=6
    DEFAULT=7
    DO=8
    DOUBLE=9
    ELSE=10
    ENUM=11
    EXTERN=12
    FLOAT=13
    FOR=14
    GOTO=15
    IF=16
    INLINE=17
    INT=18
    LONG=19
    REGISTER=20
    RESTRICT=21
    RETURN=22
    SHORT=23
    SIGNED=24
    SIZEOF=25
    STATIC=26
    STRUCT=27
    SWITCH=28
    TYPEDEF=29
    UNION=30
    UNSIGNED=31
    VOID=32
    VOLATILE=33
    WHILE=34
    BOOL_=35
    COMPLEX=36
    IMAGINERY=37
    TRUE=38
    FALSE=39
    BOOL=40
    Class=41
    BYCOPY=42
    BYREF=43
    ID=44
    IMP=45
    IN=46
    INOUT=47
    NIL=48
    NO=49
    NULL=50
    ONEWAY=51
    OUT=52
    PROTOCOL_=53
    SEL=54
    SELF=55
    SUPER=56
    YES=57
    AUTORELEASEPOOL=58
    CATCH=59
    CLASS=60
    DYNAMIC=61
    ENCODE=62
    END=63
    FINALLY=64
    IMPLEMENTATION=65
    INTERFACE=66
    IMPORT=67
    PACKAGE=68
    PROTOCOL=69
    OPTIONAL=70
    PRIVATE=71
    PROPERTY=72
    PROTECTED=73
    PUBLIC=74
    REQUIRED=75
    SELECTOR=76
    SYNCHRONIZED=77
    SYNTHESIZE=78
    THROW=79
    TRY=80
    ATOMIC=81
    NONATOMIC=82
    RETAIN=83
    ATTRIBUTE=84
    AUTORELEASING_QUALIFIER=85
    BLOCK=86
    BRIDGE=87
    BRIDGE_RETAINED=88
    BRIDGE_TRANSFER=89
    COVARIANT=90
    CONTRAVARIANT=91
    DEPRECATED=92
    KINDOF=93
    STRONG_QUALIFIER=94
    TYPEOF=95
    UNSAFE_UNRETAINED_QUALIFIER=96
    UNUSED=97
    WEAK_QUALIFIER=98
    NULL_UNSPECIFIED=99
    NULLABLE=100
    NONNULL=101
    NULL_RESETTABLE=102
    NS_INLINE=103
    NS_ENUM=104
    NS_OPTIONS=105
    ASSIGN=106
    COPY=107
    GETTER=108
    SETTER=109
    STRONG=110
    READONLY=111
    READWRITE=112
    WEAK=113
    UNSAFE_UNRETAINED=114
    IB_OUTLET=115
    IB_OUTLET_COLLECTION=116
    IB_INSPECTABLE=117
    IB_DESIGNABLE=118
    NS_ASSUME_NONNULL_BEGIN=119
    NS_ASSUME_NONNULL_END=120
    EXTERN_SUFFIX=121
    IOS_SUFFIX=122
    MAC_SUFFIX=123
    TVOS_PROHIBITED=124
    IDENTIFIER=125
    LP=126
    RP=127
    LBRACE=128
    RBRACE=129
    LBRACK=130
    RBRACK=131
    SEMI=132
    COMMA=133
    DOT=134
    STRUCTACCESS=135
    AT=136
    ASSIGNMENT=137
    GT=138
    LT=139
    BANG=140
    TILDE=141
    QUESTION=142
    COLON=143
    EQUAL=144
    LE=145
    GE=146
    NOTEQUAL=147
    AND=148
    OR=149
    INC=150
    DEC=151
    ADD=152
    SUB=153
    MUL=154
    DIV=155
    BITAND=156
    BITOR=157
    BITXOR=158
    MOD=159
    ADD_ASSIGN=160
    SUB_ASSIGN=161
    MUL_ASSIGN=162
    DIV_ASSIGN=163
    AND_ASSIGN=164
    OR_ASSIGN=165
    XOR_ASSIGN=166
    MOD_ASSIGN=167
    LSHIFT_ASSIGN=168
    RSHIFT_ASSIGN=169
    ELIPSIS=170
    CHARACTER_LITERAL=171
    STRING_START=172
    HEX_LITERAL=173
    OCTAL_LITERAL=174
    BINARY_LITERAL=175
    DECIMAL_LITERAL=176
    FLOATING_POINT_LITERAL=177
    WS=178
    MULTI_COMMENT=179
    SINGLE_COMMENT=180
    BACKSLASH=181
    SHARP=182
    STRING_NEWLINE=183
    STRING_END=184
    STRING_VALUE=185
    DIRECTIVE_IMPORT=186
    DIRECTIVE_INCLUDE=187
    DIRECTIVE_PRAGMA=188
    DIRECTIVE_DEFINE=189
    DIRECTIVE_DEFINED=190
    DIRECTIVE_IF=191
    DIRECTIVE_ELIF=192
    DIRECTIVE_ELSE=193
    DIRECTIVE_UNDEF=194
    DIRECTIVE_IFDEF=195
    DIRECTIVE_IFNDEF=196
    DIRECTIVE_ENDIF=197
    DIRECTIVE_TRUE=198
    DIRECTIVE_FALSE=199
    DIRECTIVE_ERROR=200
    DIRECTIVE_WARNING=201
    DIRECTIVE_BANG=202
    DIRECTIVE_LP=203
    DIRECTIVE_RP=204
    DIRECTIVE_EQUAL=205
    DIRECTIVE_NOTEQUAL=206
    DIRECTIVE_AND=207
    DIRECTIVE_OR=208
    DIRECTIVE_LT=209
    DIRECTIVE_GT=210
    DIRECTIVE_LE=211
    DIRECTIVE_GE=212
    DIRECTIVE_STRING=213
    DIRECTIVE_ID=214
    DIRECTIVE_DECIMAL_LITERAL=215
    DIRECTIVE_FLOAT=216
    DIRECTIVE_NEWLINE=217
    DIRECTIVE_MULTI_COMMENT=218
    DIRECTIVE_SINGLE_COMMENT=219
    DIRECTIVE_BACKSLASH_NEWLINE=220
    DIRECTIVE_TEXT_NEWLINE=221
    DIRECTIVE_TEXT=222

    def __init__(self, input, output=sys.stdout):
        super(ObjectiveCPreprocessorParser, self).__init__(input, output=output)
        self.checkVersion("4.7.1")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None



    class DirectiveContext(ParserRuleContext):

        def __init__(self, parser, parent=None, invokingState=-1):
            super(ObjectiveCPreprocessorParser.DirectiveContext, self).__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return ObjectiveCPreprocessorParser.RULE_directive

     
        def copyFrom(self, ctx):
            super(ObjectiveCPreprocessorParser.DirectiveContext, self).copyFrom(ctx)



    class PreprocessorDefContext(DirectiveContext):

        def __init__(self, parser, ctx): # actually a ObjectiveCPreprocessorParser.DirectiveContext)
            super(ObjectiveCPreprocessorParser.PreprocessorDefContext, self).__init__(parser)
            self.copyFrom(ctx)

        def SHARP(self):
            return self.getToken(ObjectiveCPreprocessorParser.SHARP, 0)
        def DIRECTIVE_IFDEF(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_IFDEF, 0)
        def DIRECTIVE_ID(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_ID, 0)
        def DIRECTIVE_IFNDEF(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_IFNDEF, 0)
        def DIRECTIVE_UNDEF(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_UNDEF, 0)

        def enterRule(self, listener):
            if hasattr(listener, "enterPreprocessorDef"):
                listener.enterPreprocessorDef(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitPreprocessorDef"):
                listener.exitPreprocessorDef(self)


    class PreprocessorErrorContext(DirectiveContext):

        def __init__(self, parser, ctx): # actually a ObjectiveCPreprocessorParser.DirectiveContext)
            super(ObjectiveCPreprocessorParser.PreprocessorErrorContext, self).__init__(parser)
            self.copyFrom(ctx)

        def SHARP(self):
            return self.getToken(ObjectiveCPreprocessorParser.SHARP, 0)
        def DIRECTIVE_ERROR(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_ERROR, 0)
        def directiveText(self):
            return self.getTypedRuleContext(ObjectiveCPreprocessorParser.DirectiveTextContext,0)


        def enterRule(self, listener):
            if hasattr(listener, "enterPreprocessorError"):
                listener.enterPreprocessorError(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitPreprocessorError"):
                listener.exitPreprocessorError(self)


    class PreprocessorConditionalContext(DirectiveContext):

        def __init__(self, parser, ctx): # actually a ObjectiveCPreprocessorParser.DirectiveContext)
            super(ObjectiveCPreprocessorParser.PreprocessorConditionalContext, self).__init__(parser)
            self.copyFrom(ctx)

        def SHARP(self):
            return self.getToken(ObjectiveCPreprocessorParser.SHARP, 0)
        def DIRECTIVE_IF(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_IF, 0)
        def preprocessorExpression(self):
            return self.getTypedRuleContext(ObjectiveCPreprocessorParser.PreprocessorExpressionContext,0)

        def DIRECTIVE_ELIF(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_ELIF, 0)
        def DIRECTIVE_ELSE(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_ELSE, 0)
        def DIRECTIVE_ENDIF(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_ENDIF, 0)

        def enterRule(self, listener):
            if hasattr(listener, "enterPreprocessorConditional"):
                listener.enterPreprocessorConditional(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitPreprocessorConditional"):
                listener.exitPreprocessorConditional(self)


    class PreprocessorImportContext(DirectiveContext):

        def __init__(self, parser, ctx): # actually a ObjectiveCPreprocessorParser.DirectiveContext)
            super(ObjectiveCPreprocessorParser.PreprocessorImportContext, self).__init__(parser)
            self.copyFrom(ctx)

        def SHARP(self):
            return self.getToken(ObjectiveCPreprocessorParser.SHARP, 0)
        def directiveText(self):
            return self.getTypedRuleContext(ObjectiveCPreprocessorParser.DirectiveTextContext,0)

        def DIRECTIVE_IMPORT(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_IMPORT, 0)
        def DIRECTIVE_INCLUDE(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_INCLUDE, 0)

        def enterRule(self, listener):
            if hasattr(listener, "enterPreprocessorImport"):
                listener.enterPreprocessorImport(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitPreprocessorImport"):
                listener.exitPreprocessorImport(self)


    class PreprocessorPragmaContext(DirectiveContext):

        def __init__(self, parser, ctx): # actually a ObjectiveCPreprocessorParser.DirectiveContext)
            super(ObjectiveCPreprocessorParser.PreprocessorPragmaContext, self).__init__(parser)
            self.copyFrom(ctx)

        def SHARP(self):
            return self.getToken(ObjectiveCPreprocessorParser.SHARP, 0)
        def DIRECTIVE_PRAGMA(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_PRAGMA, 0)
        def directiveText(self):
            return self.getTypedRuleContext(ObjectiveCPreprocessorParser.DirectiveTextContext,0)


        def enterRule(self, listener):
            if hasattr(listener, "enterPreprocessorPragma"):
                listener.enterPreprocessorPragma(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitPreprocessorPragma"):
                listener.exitPreprocessorPragma(self)


    class PreprocessorDefineContext(DirectiveContext):

        def __init__(self, parser, ctx): # actually a ObjectiveCPreprocessorParser.DirectiveContext)
            super(ObjectiveCPreprocessorParser.PreprocessorDefineContext, self).__init__(parser)
            self.copyFrom(ctx)

        def SHARP(self):
            return self.getToken(ObjectiveCPreprocessorParser.SHARP, 0)
        def DIRECTIVE_DEFINE(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_DEFINE, 0)
        def DIRECTIVE_ID(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_ID, 0)
        def directiveText(self):
            return self.getTypedRuleContext(ObjectiveCPreprocessorParser.DirectiveTextContext,0)


        def enterRule(self, listener):
            if hasattr(listener, "enterPreprocessorDefine"):
                listener.enterPreprocessorDefine(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitPreprocessorDefine"):
                listener.exitPreprocessorDefine(self)


    class PreprocessorWarningContext(DirectiveContext):

        def __init__(self, parser, ctx): # actually a ObjectiveCPreprocessorParser.DirectiveContext)
            super(ObjectiveCPreprocessorParser.PreprocessorWarningContext, self).__init__(parser)
            self.copyFrom(ctx)

        def SHARP(self):
            return self.getToken(ObjectiveCPreprocessorParser.SHARP, 0)
        def DIRECTIVE_WARNING(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_WARNING, 0)
        def directiveText(self):
            return self.getTypedRuleContext(ObjectiveCPreprocessorParser.DirectiveTextContext,0)


        def enterRule(self, listener):
            if hasattr(listener, "enterPreprocessorWarning"):
                listener.enterPreprocessorWarning(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitPreprocessorWarning"):
                listener.exitPreprocessorWarning(self)



    def directive(self):

        localctx = ObjectiveCPreprocessorParser.DirectiveContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_directive)
        self._la = 0 # Token type
        try:
            self.state = 43
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,1,self._ctx)
            if la_ == 1:
                localctx = ObjectiveCPreprocessorParser.PreprocessorImportContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 6
                self.match(ObjectiveCPreprocessorParser.SHARP)
                self.state = 7
                _la = self._input.LA(1)
                if not(_la==ObjectiveCPreprocessorParser.DIRECTIVE_IMPORT or _la==ObjectiveCPreprocessorParser.DIRECTIVE_INCLUDE):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 8
                self.directiveText()
                pass

            elif la_ == 2:
                localctx = ObjectiveCPreprocessorParser.PreprocessorConditionalContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 9
                self.match(ObjectiveCPreprocessorParser.SHARP)
                self.state = 10
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_IF)
                self.state = 11
                self.preprocessorExpression(0)
                pass

            elif la_ == 3:
                localctx = ObjectiveCPreprocessorParser.PreprocessorConditionalContext(self, localctx)
                self.enterOuterAlt(localctx, 3)
                self.state = 12
                self.match(ObjectiveCPreprocessorParser.SHARP)
                self.state = 13
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_ELIF)
                self.state = 14
                self.preprocessorExpression(0)
                pass

            elif la_ == 4:
                localctx = ObjectiveCPreprocessorParser.PreprocessorConditionalContext(self, localctx)
                self.enterOuterAlt(localctx, 4)
                self.state = 15
                self.match(ObjectiveCPreprocessorParser.SHARP)
                self.state = 16
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_ELSE)
                pass

            elif la_ == 5:
                localctx = ObjectiveCPreprocessorParser.PreprocessorConditionalContext(self, localctx)
                self.enterOuterAlt(localctx, 5)
                self.state = 17
                self.match(ObjectiveCPreprocessorParser.SHARP)
                self.state = 18
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_ENDIF)
                pass

            elif la_ == 6:
                localctx = ObjectiveCPreprocessorParser.PreprocessorDefContext(self, localctx)
                self.enterOuterAlt(localctx, 6)
                self.state = 19
                self.match(ObjectiveCPreprocessorParser.SHARP)
                self.state = 20
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_IFDEF)
                self.state = 21
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_ID)
                pass

            elif la_ == 7:
                localctx = ObjectiveCPreprocessorParser.PreprocessorDefContext(self, localctx)
                self.enterOuterAlt(localctx, 7)
                self.state = 22
                self.match(ObjectiveCPreprocessorParser.SHARP)
                self.state = 23
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_IFNDEF)
                self.state = 24
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_ID)
                pass

            elif la_ == 8:
                localctx = ObjectiveCPreprocessorParser.PreprocessorDefContext(self, localctx)
                self.enterOuterAlt(localctx, 8)
                self.state = 25
                self.match(ObjectiveCPreprocessorParser.SHARP)
                self.state = 26
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_UNDEF)
                self.state = 27
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_ID)
                pass

            elif la_ == 9:
                localctx = ObjectiveCPreprocessorParser.PreprocessorPragmaContext(self, localctx)
                self.enterOuterAlt(localctx, 9)
                self.state = 28
                self.match(ObjectiveCPreprocessorParser.SHARP)
                self.state = 29
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_PRAGMA)
                self.state = 30
                self.directiveText()
                pass

            elif la_ == 10:
                localctx = ObjectiveCPreprocessorParser.PreprocessorErrorContext(self, localctx)
                self.enterOuterAlt(localctx, 10)
                self.state = 31
                self.match(ObjectiveCPreprocessorParser.SHARP)
                self.state = 32
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_ERROR)
                self.state = 33
                self.directiveText()
                pass

            elif la_ == 11:
                localctx = ObjectiveCPreprocessorParser.PreprocessorWarningContext(self, localctx)
                self.enterOuterAlt(localctx, 11)
                self.state = 34
                self.match(ObjectiveCPreprocessorParser.SHARP)
                self.state = 35
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_WARNING)
                self.state = 36
                self.directiveText()
                pass

            elif la_ == 12:
                localctx = ObjectiveCPreprocessorParser.PreprocessorDefineContext(self, localctx)
                self.enterOuterAlt(localctx, 12)
                self.state = 37
                self.match(ObjectiveCPreprocessorParser.SHARP)
                self.state = 38
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_DEFINE)
                self.state = 39
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_ID)
                self.state = 41
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==ObjectiveCPreprocessorParser.DIRECTIVE_TEXT_NEWLINE or _la==ObjectiveCPreprocessorParser.DIRECTIVE_TEXT:
                    self.state = 40
                    self.directiveText()


                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class DirectiveTextContext(ParserRuleContext):

        def __init__(self, parser, parent=None, invokingState=-1):
            super(ObjectiveCPreprocessorParser.DirectiveTextContext, self).__init__(parent, invokingState)
            self.parser = parser

        def DIRECTIVE_TEXT(self, i=None):
            if i is None:
                return self.getTokens(ObjectiveCPreprocessorParser.DIRECTIVE_TEXT)
            else:
                return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_TEXT, i)

        def DIRECTIVE_TEXT_NEWLINE(self, i=None):
            if i is None:
                return self.getTokens(ObjectiveCPreprocessorParser.DIRECTIVE_TEXT_NEWLINE)
            else:
                return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_TEXT_NEWLINE, i)

        def getRuleIndex(self):
            return ObjectiveCPreprocessorParser.RULE_directiveText

        def enterRule(self, listener):
            if hasattr(listener, "enterDirectiveText"):
                listener.enterDirectiveText(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitDirectiveText"):
                listener.exitDirectiveText(self)




    def directiveText(self):

        localctx = ObjectiveCPreprocessorParser.DirectiveTextContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_directiveText)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 46 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 45
                _la = self._input.LA(1)
                if not(_la==ObjectiveCPreprocessorParser.DIRECTIVE_TEXT_NEWLINE or _la==ObjectiveCPreprocessorParser.DIRECTIVE_TEXT):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 48 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==ObjectiveCPreprocessorParser.DIRECTIVE_TEXT_NEWLINE or _la==ObjectiveCPreprocessorParser.DIRECTIVE_TEXT):
                    break

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class PreprocessorExpressionContext(ParserRuleContext):

        def __init__(self, parser, parent=None, invokingState=-1):
            super(ObjectiveCPreprocessorParser.PreprocessorExpressionContext, self).__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return ObjectiveCPreprocessorParser.RULE_preprocessorExpression

     
        def copyFrom(self, ctx):
            super(ObjectiveCPreprocessorParser.PreprocessorExpressionContext, self).copyFrom(ctx)


    class PreprocessorParenthesisContext(PreprocessorExpressionContext):

        def __init__(self, parser, ctx): # actually a ObjectiveCPreprocessorParser.PreprocessorExpressionContext)
            super(ObjectiveCPreprocessorParser.PreprocessorParenthesisContext, self).__init__(parser)
            self.copyFrom(ctx)

        def DIRECTIVE_LP(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_LP, 0)
        def preprocessorExpression(self):
            return self.getTypedRuleContext(ObjectiveCPreprocessorParser.PreprocessorExpressionContext,0)

        def DIRECTIVE_RP(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_RP, 0)

        def enterRule(self, listener):
            if hasattr(listener, "enterPreprocessorParenthesis"):
                listener.enterPreprocessorParenthesis(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitPreprocessorParenthesis"):
                listener.exitPreprocessorParenthesis(self)


    class PreprocessorNotContext(PreprocessorExpressionContext):

        def __init__(self, parser, ctx): # actually a ObjectiveCPreprocessorParser.PreprocessorExpressionContext)
            super(ObjectiveCPreprocessorParser.PreprocessorNotContext, self).__init__(parser)
            self.copyFrom(ctx)

        def DIRECTIVE_BANG(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_BANG, 0)
        def preprocessorExpression(self):
            return self.getTypedRuleContext(ObjectiveCPreprocessorParser.PreprocessorExpressionContext,0)


        def enterRule(self, listener):
            if hasattr(listener, "enterPreprocessorNot"):
                listener.enterPreprocessorNot(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitPreprocessorNot"):
                listener.exitPreprocessorNot(self)


    class PreprocessorBinaryContext(PreprocessorExpressionContext):

        def __init__(self, parser, ctx): # actually a ObjectiveCPreprocessorParser.PreprocessorExpressionContext)
            super(ObjectiveCPreprocessorParser.PreprocessorBinaryContext, self).__init__(parser)
            self.op = None # Token
            self.copyFrom(ctx)

        def preprocessorExpression(self, i=None):
            if i is None:
                return self.getTypedRuleContexts(ObjectiveCPreprocessorParser.PreprocessorExpressionContext)
            else:
                return self.getTypedRuleContext(ObjectiveCPreprocessorParser.PreprocessorExpressionContext,i)

        def DIRECTIVE_EQUAL(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_EQUAL, 0)
        def DIRECTIVE_NOTEQUAL(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_NOTEQUAL, 0)
        def DIRECTIVE_AND(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_AND, 0)
        def DIRECTIVE_OR(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_OR, 0)
        def DIRECTIVE_LT(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_LT, 0)
        def DIRECTIVE_GT(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_GT, 0)
        def DIRECTIVE_LE(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_LE, 0)
        def DIRECTIVE_GE(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_GE, 0)

        def enterRule(self, listener):
            if hasattr(listener, "enterPreprocessorBinary"):
                listener.enterPreprocessorBinary(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitPreprocessorBinary"):
                listener.exitPreprocessorBinary(self)


    class PreprocessorConstantContext(PreprocessorExpressionContext):

        def __init__(self, parser, ctx): # actually a ObjectiveCPreprocessorParser.PreprocessorExpressionContext)
            super(ObjectiveCPreprocessorParser.PreprocessorConstantContext, self).__init__(parser)
            self.copyFrom(ctx)

        def DIRECTIVE_TRUE(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_TRUE, 0)
        def DIRECTIVE_FALSE(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_FALSE, 0)
        def DIRECTIVE_DECIMAL_LITERAL(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_DECIMAL_LITERAL, 0)
        def DIRECTIVE_STRING(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_STRING, 0)

        def enterRule(self, listener):
            if hasattr(listener, "enterPreprocessorConstant"):
                listener.enterPreprocessorConstant(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitPreprocessorConstant"):
                listener.exitPreprocessorConstant(self)


    class PreprocessorConditionalSymbolContext(PreprocessorExpressionContext):

        def __init__(self, parser, ctx): # actually a ObjectiveCPreprocessorParser.PreprocessorExpressionContext)
            super(ObjectiveCPreprocessorParser.PreprocessorConditionalSymbolContext, self).__init__(parser)
            self.copyFrom(ctx)

        def DIRECTIVE_ID(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_ID, 0)
        def DIRECTIVE_LP(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_LP, 0)
        def preprocessorExpression(self):
            return self.getTypedRuleContext(ObjectiveCPreprocessorParser.PreprocessorExpressionContext,0)

        def DIRECTIVE_RP(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_RP, 0)

        def enterRule(self, listener):
            if hasattr(listener, "enterPreprocessorConditionalSymbol"):
                listener.enterPreprocessorConditionalSymbol(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitPreprocessorConditionalSymbol"):
                listener.exitPreprocessorConditionalSymbol(self)


    class PreprocessorDefinedContext(PreprocessorExpressionContext):

        def __init__(self, parser, ctx): # actually a ObjectiveCPreprocessorParser.PreprocessorExpressionContext)
            super(ObjectiveCPreprocessorParser.PreprocessorDefinedContext, self).__init__(parser)
            self.copyFrom(ctx)

        def DIRECTIVE_DEFINED(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_DEFINED, 0)
        def DIRECTIVE_ID(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_ID, 0)
        def DIRECTIVE_LP(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_LP, 0)
        def DIRECTIVE_RP(self):
            return self.getToken(ObjectiveCPreprocessorParser.DIRECTIVE_RP, 0)

        def enterRule(self, listener):
            if hasattr(listener, "enterPreprocessorDefined"):
                listener.enterPreprocessorDefined(self)

        def exitRule(self, listener):
            if hasattr(listener, "exitPreprocessorDefined"):
                listener.exitPreprocessorDefined(self)



    def preprocessorExpression(self, _p=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = ObjectiveCPreprocessorParser.PreprocessorExpressionContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 4
        self.enterRecursionRule(localctx, 4, self.RULE_preprocessorExpression, _p)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 75
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [ObjectiveCPreprocessorParser.DIRECTIVE_TRUE]:
                localctx = ObjectiveCPreprocessorParser.PreprocessorConstantContext(self, localctx)
                self._ctx = localctx
                _prevctx = localctx

                self.state = 51
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_TRUE)
                pass
            elif token in [ObjectiveCPreprocessorParser.DIRECTIVE_FALSE]:
                localctx = ObjectiveCPreprocessorParser.PreprocessorConstantContext(self, localctx)
                self._ctx = localctx
                _prevctx = localctx
                self.state = 52
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_FALSE)
                pass
            elif token in [ObjectiveCPreprocessorParser.DIRECTIVE_DECIMAL_LITERAL]:
                localctx = ObjectiveCPreprocessorParser.PreprocessorConstantContext(self, localctx)
                self._ctx = localctx
                _prevctx = localctx
                self.state = 53
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_DECIMAL_LITERAL)
                pass
            elif token in [ObjectiveCPreprocessorParser.DIRECTIVE_STRING]:
                localctx = ObjectiveCPreprocessorParser.PreprocessorConstantContext(self, localctx)
                self._ctx = localctx
                _prevctx = localctx
                self.state = 54
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_STRING)
                pass
            elif token in [ObjectiveCPreprocessorParser.DIRECTIVE_ID]:
                localctx = ObjectiveCPreprocessorParser.PreprocessorConditionalSymbolContext(self, localctx)
                self._ctx = localctx
                _prevctx = localctx
                self.state = 55
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_ID)
                self.state = 60
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,3,self._ctx)
                if la_ == 1:
                    self.state = 56
                    self.match(ObjectiveCPreprocessorParser.DIRECTIVE_LP)
                    self.state = 57
                    self.preprocessorExpression(0)
                    self.state = 58
                    self.match(ObjectiveCPreprocessorParser.DIRECTIVE_RP)


                pass
            elif token in [ObjectiveCPreprocessorParser.DIRECTIVE_LP]:
                localctx = ObjectiveCPreprocessorParser.PreprocessorParenthesisContext(self, localctx)
                self._ctx = localctx
                _prevctx = localctx
                self.state = 62
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_LP)
                self.state = 63
                self.preprocessorExpression(0)
                self.state = 64
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_RP)
                pass
            elif token in [ObjectiveCPreprocessorParser.DIRECTIVE_BANG]:
                localctx = ObjectiveCPreprocessorParser.PreprocessorNotContext(self, localctx)
                self._ctx = localctx
                _prevctx = localctx
                self.state = 66
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_BANG)
                self.state = 67
                self.preprocessorExpression(6)
                pass
            elif token in [ObjectiveCPreprocessorParser.DIRECTIVE_DEFINED]:
                localctx = ObjectiveCPreprocessorParser.PreprocessorDefinedContext(self, localctx)
                self._ctx = localctx
                _prevctx = localctx
                self.state = 68
                self.match(ObjectiveCPreprocessorParser.DIRECTIVE_DEFINED)
                self.state = 73
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [ObjectiveCPreprocessorParser.DIRECTIVE_ID]:
                    self.state = 69
                    self.match(ObjectiveCPreprocessorParser.DIRECTIVE_ID)
                    pass
                elif token in [ObjectiveCPreprocessorParser.DIRECTIVE_LP]:
                    self.state = 70
                    self.match(ObjectiveCPreprocessorParser.DIRECTIVE_LP)
                    self.state = 71
                    self.match(ObjectiveCPreprocessorParser.DIRECTIVE_ID)
                    self.state = 72
                    self.match(ObjectiveCPreprocessorParser.DIRECTIVE_RP)
                    pass
                else:
                    raise NoViableAltException(self)

                pass
            else:
                raise NoViableAltException(self)

            self._ctx.stop = self._input.LT(-1)
            self.state = 91
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,7,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    self.state = 89
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,6,self._ctx)
                    if la_ == 1:
                        localctx = ObjectiveCPreprocessorParser.PreprocessorBinaryContext(self, ObjectiveCPreprocessorParser.PreprocessorExpressionContext(self, _parentctx, _parentState))
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_preprocessorExpression)
                        self.state = 77
                        if not self.precpred(self._ctx, 5):
                            from QBCodeSpecs.antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 5)")
                        self.state = 78
                        localctx.op = self._input.LT(1)
                        _la = self._input.LA(1)
                        if not(_la==ObjectiveCPreprocessorParser.DIRECTIVE_EQUAL or _la==ObjectiveCPreprocessorParser.DIRECTIVE_NOTEQUAL):
                            localctx.op = self._errHandler.recoverInline(self)
                        else:
                            self._errHandler.reportMatch(self)
                            self.consume()
                        self.state = 79
                        self.preprocessorExpression(6)
                        pass

                    elif la_ == 2:
                        localctx = ObjectiveCPreprocessorParser.PreprocessorBinaryContext(self, ObjectiveCPreprocessorParser.PreprocessorExpressionContext(self, _parentctx, _parentState))
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_preprocessorExpression)
                        self.state = 80
                        if not self.precpred(self._ctx, 4):
                            from QBCodeSpecs.antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 4)")
                        self.state = 81
                        localctx.op = self.match(ObjectiveCPreprocessorParser.DIRECTIVE_AND)
                        self.state = 82
                        self.preprocessorExpression(5)
                        pass

                    elif la_ == 3:
                        localctx = ObjectiveCPreprocessorParser.PreprocessorBinaryContext(self, ObjectiveCPreprocessorParser.PreprocessorExpressionContext(self, _parentctx, _parentState))
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_preprocessorExpression)
                        self.state = 83
                        if not self.precpred(self._ctx, 3):
                            from QBCodeSpecs.antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 3)")
                        self.state = 84
                        localctx.op = self.match(ObjectiveCPreprocessorParser.DIRECTIVE_OR)
                        self.state = 85
                        self.preprocessorExpression(4)
                        pass

                    elif la_ == 4:
                        localctx = ObjectiveCPreprocessorParser.PreprocessorBinaryContext(self, ObjectiveCPreprocessorParser.PreprocessorExpressionContext(self, _parentctx, _parentState))
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_preprocessorExpression)
                        self.state = 86
                        if not self.precpred(self._ctx, 2):
                            from QBCodeSpecs.antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 2)")
                        self.state = 87
                        localctx.op = self._input.LT(1)
                        _la = self._input.LA(1)
                        if not(((((_la - 209)) & ~0x3f) == 0 and ((1 << (_la - 209)) & ((1 << (ObjectiveCPreprocessorParser.DIRECTIVE_LT - 209)) | (1 << (ObjectiveCPreprocessorParser.DIRECTIVE_GT - 209)) | (1 << (ObjectiveCPreprocessorParser.DIRECTIVE_LE - 209)) | (1 << (ObjectiveCPreprocessorParser.DIRECTIVE_GE - 209)))) != 0)):
                            localctx.op = self._errHandler.recoverInline(self)
                        else:
                            self._errHandler.reportMatch(self)
                            self.consume()
                        self.state = 88
                        self.preprocessorExpression(3)
                        pass

             
                self.state = 93
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,7,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx



    def sempred(self, localctx, ruleIndex, predIndex):
        if self._predicates == None:
            self._predicates = dict()
        self._predicates[2] = self.preprocessorExpression_sempred
        pred = self._predicates.get(ruleIndex, None)
        if pred is None:
            raise Exception("No predicate with index:" + str(ruleIndex))
        else:
            return pred(localctx, predIndex)

    def preprocessorExpression_sempred(self, localctx, predIndex):
            if predIndex == 0:
                return self.precpred(self._ctx, 5)
         

            if predIndex == 1:
                return self.precpred(self._ctx, 4)
         

            if predIndex == 2:
                return self.precpred(self._ctx, 3)
         

            if predIndex == 3:
                return self.precpred(self._ctx, 2)
         







#!/usr/bin/env python
# -coding:utf-8
import sys
from QBCodeSpecs.objcRecoginition.ObjectiveCParserListener import *
from QBCodeSpecs.driver.ViolationDataMananger import *
from QBCodeSpecs.driver.Violation import *
import re
import linecache


class InstancetypeInitRuleListener(ObjectiveCParserListener):

	def __init__(self, violation):
		super(InstancetypeInitRuleListener,self).__init__()
		self.violation = violation
		self.startline = 0
		self.startcolumn = 0


	def send_violation(self, startline, startcolumn):

		path = self.violation.path
		configInfo = self.violation.configinfo

		violInfo = Violation(path, configInfo)
		violInfo.startline = startline
		violInfo.startcolumn = startcolumn

		# violInfo.set_rulename("")
		violInfo.set_rulecategory("初始化返回值类型")
		codeStr = linecache.getline(path,self.startline)
		# print "source code:" + codeStr
		violInfo.message = "方法定义 " + codeStr + " 返回值的类型应为instancetype"
		#"Conforming to regular expressions:" + "\"" + self.regex + "\"" 

		democode = "- (instancetype)init"
		violInfo.set_democode(democode)
		noncompliant = "- (id)init"
		violInfo.set_noncompliant_code(noncompliant)
		violationDataMananger.addViolation(violInfo)

	def enterImplementationDefinitionList(self, ctx):


		classDefList = ctx.classMethodDefinition()
		for classDef in classDefList:
			methodSel = classDef.methodDefinition().methodSelector()

			selectorStr = methodSel.getText()
			print methodSel.getText()
			if selectorStr.startswith("init"):
				
				
				
				methodType = classDef.methodDefinition().methodType()
				typeName = methodType.typeName().getText()
				# print "---typeName:" + typeName
				if not "instancetype" in typeName:
					# print "ok"
					token = methodSel.start
					self.startline = token.line
					self.startcolumn = token.column
					self.send_violation(token.line, token.column)
				# else:
				# 	print "fail"


		instanceDefList = ctx.instanceMethodDefinition()
		for instanceDef in instanceDefList:
			methodSel = instanceDef.methodDefinition().methodSelector()
			selectorStr = methodSel.getText()
			# token = methodSel.start
			# print "line:" + str(token.line) + "column:" + str(token.column)
			print methodSel.getText()
			if selectorStr.startswith("init"):
				token = methodSel.start
				self.startline = token.line
				self.startcolumn = token.column
				# foundShared = True
				methodType = instanceDef.methodDefinition().methodType()
				typeName = methodType.typeName().getText()
				# print "---typeName:" + typeName
				if not "instancetype" in typeName:
					# print "ok"
					token = methodSel.start
					self.startline = token.line
					self.startcolumn = token.column
					self.send_violation(token.line, token.column)
				# else:
				# 	print "fail"





#!/usr/bin/env python
# -coding:utf-8
import sys
from QBCodeSpecs.objcRecoginition import ObjectiveCParserListener
from QBCodeSpecs.driver.ViolationDataMananger import *
from QBCodeSpecs.driver.Violation import *
import re

class MethodNameRuleListener(ObjectiveCParserListener.ObjectiveCParserListener):
	def __init__(self, violation):
		super(MethodNameRuleListener,self).__init__()
		self.violation = violation
		self.regex = self.violation.configinfo["Regex"]


	def send_violation(self, startline, startcolumn, selector):

		path = self.violation.path
		configInfo = self.violation.configinfo

		violInfo = Violation(path, configInfo)
		violInfo.startline = startline
		violInfo.startcolumn = startcolumn
		# violInfo.message = "Conforming to regular expressions:" + "\"" + self.regex + "\""
		violInfo.message =  "方法名 "+ "\"" + selector + "\"" + " 不符合正则 " + "\"" + self.regex + "\""
		violInfo.set_priority("2")
		violInfo.set_rulename("Name")
		violInfo.set_rulecategory("方法名定义")
		violInfo.set_democode("- (void)methodSeletor")
		violInfo.set_noncompliant_code("- (void)MethodSeletor \n\
- (void)_methodSeletor")
		violationDataMananger.addViolation(violInfo)


	def checkSelector(self, selector):
		if len(self.regex) > 0:
			if re.match(self.regex, selector.getText()):
			    print 'ok'
			else:
				token = selector.start
				print "error selector:" + "\"" + selector.getText() + "\"" + "------" + "[" + str(token.line) + ":" + str(token.column) + "]"
				self.send_violation(token.line, token.column, selector.getText())
				print 'failed'

	def exitMethodSelector(self, ctx):

		selector = ctx.selector()
		if selector != None:
			# print("selector:"+selector.getText());
			self.checkSelector(selector)


		keywordDeclarator = ctx.keywordDeclarator()
		if isinstance(keywordDeclarator, list):

			for index in range(len(keywordDeclarator)):
				selector = keywordDeclarator[index].selector()
				self.checkSelector(selector)
			



# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.

class ATNDeserializationOptions(object):

    defaultOptions = None

    def __init__(self, copyFrom = None):
        self.readOnly = False
        self.verifyATN = True if copyFrom is None else copyFrom.verifyATN
        self.generateRuleBypassTransitions = False if copyFrom is None else copyFrom.generateRuleBypassTransitions

    def __setattr__(self, key, value):
        if key!="readOnly" and self.readOnly:
            raise Exception("The object is read only.")
        super(type(self), self).__setattr__(key,value)

ATNDeserializationOptions.defaultOptions = ATNDeserializationOptions()
ATNDeserializationOptions.defaultOptions.readOnly = True




# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.
#

from io import StringIO

def str_collection(val, begin, end):
    with StringIO() as buf:
        buf.write(begin)
        first = True
        for item in val:
            if not first:
                buf.write(u', ')
            buf.write(unicode(item))
            first = False
        buf.write(end)
        return buf.getvalue()

def str_list(val):
    return str_collection(val, u'[', u']')

def str_set(val):
    return str_collection(val, u'{', u'}')

def escapeWhitespace(s, escapeSpaces):
    with StringIO() as buf:
        for c in s:
            if c==' ' and escapeSpaces:
                buf.write(u'\u00B7')
            elif c=='\t':
                buf.write(u"\\t")
            elif c=='\n':
                buf.write(u"\\n")
            elif c=='\r':
                buf.write(u"\\r")
            else:
                buf.write(unicode(c))
        return buf.getvalue()



#!/usr/bin/env python
# -coding:utf-8
import sys
from QBCodeSpecs.objcRecoginition.ObjectiveCParserListener import *
from QBCodeSpecs.driver.ViolationDataMananger import *
from QBCodeSpecs.driver.Violation import *
import re
import linecache


class CategoryMethodNameRuleListener(ObjectiveCParserListener):

	def __init__(self, violation):
		super(CategoryMethodNameRuleListener,self).__init__()
		self.violation = violation
		self.startline = 0
		self.startcolumn = 0


	def send_violation(self, startline, startcolumn):

		path = self.violation.path
		configInfo = self.violation.configinfo

		violInfo = Violation(path, configInfo)
		violInfo.startline = startline
		violInfo.startcolumn = startcolumn

		# violInfo.set_rulename("")
		violInfo.set_rulecategory("类别方法名定义")
		codeStr = linecache.getline(path, startline)
		# print "source code:" + codeStr
		violInfo.message = "类别方法名 " + codeStr +" 的定义需要包含前缀"
		# "方法定义 " + codeStr + " 返回值的类型应为instancetype"
		#"Conforming to regular expressions:" + "\"" + self.regex + "\"" 

		democode = "@interface NSData (ZOCTimeExtensions)\n\
- (NSString *)zoc_timeAgoShort;\n\
@end"
		violInfo.set_democode(democode)
		noncompliant = "@interface NSData (ZOCTimeExtensions)\n\
- (NSString *)timeAgoShort;\n\
@end"
		violInfo.set_noncompliant_code(noncompliant)
		violationDataMananger.addViolation(violInfo)

	def exitCategoryImplementation(self, ctx):

		identifier = ctx.identifier()
		# print "identifier--" + identifier.getText()
		identifierStr = identifier.getText()
		prefix = ""
		if len(identifierStr) > 3:
			prefix = identifierStr[0:3]
		if len(prefix) == 0:
			return

		prefix = prefix.lower() + "_"
		# print "prefix--" + prefix

		implementationDefinitionContext = ctx.implementationDefinitionList()

		classDefList = implementationDefinitionContext.classMethodDefinition()
		for classDef in classDefList:
			methodSel = classDef.methodDefinition().methodSelector()

			selectorStr = methodSel.getText()
			# print "class=" + methodSel.getText()
			if not selectorStr.startswith(prefix):
				token = methodSel.start
				startline = token.line
				startcolumn = token.column
				self.send_violation(startline, startcolumn)

				


		instanceDefList = implementationDefinitionContext.instanceMethodDefinition()
		for instanceDef in instanceDefList:
			methodSel = instanceDef.methodDefinition().methodSelector()
			selectorStr = methodSel.getText()
			# print "instance=" + methodSel.getText()
			if not selectorStr.startswith(prefix):
				token = methodSel.start
				startline = token.line
				startcolumn = token.column
				self.send_violation(startline, startcolumn)
				





#
# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.
#

#
#  This is an InputStream that is loaded from stdin all at once
#  when you construct the object.
#

import codecs
import sys
from QBCodeSpecs.antlr4.InputStream import InputStream

class StdinStream(InputStream):

	def __init__(self, encoding='ascii', errors='strict'):
		bytes = sys.stdin.read()
		data = codecs.decode(bytes, encoding, errors)
		super(type(self), self).__init__(data)



#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: paulineli
# Mail: 109926082@qq.com
# Created Time:  2018-06-14 18:14:34 AM
#############################################


from setuptools import setup, find_packages

setup(
    name = "QBCodeSpecs",
    version = "0.0.1",
    keywords = ["pip", "code", "specs"],
    description = "QBCodeSpecs",
    long_description = "eds sdk for python",
    license = "MIT Licence",

    url = "https://browser.qq.com/ipad/pc.html",
    author = "paulineli",
    author_email = "109926082@qq.com",

    packages = find_packages(),
    include_package_data = True,
    platforms = "any",
    # install_requires = []
)


#
# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.
#

class LexerActionType(object):

    CHANNEL = 0     #The type of a {@link LexerChannelAction} action.
    CUSTOM = 1      #The type of a {@link LexerCustomAction} action.
    MODE = 2        #The type of a {@link LexerModeAction} action.
    MORE = 3        #The type of a {@link LexerMoreAction} action.
    POP_MODE = 4    #The type of a {@link LexerPopModeAction} action.
    PUSH_MODE = 5   #The type of a {@link LexerPushModeAction} action.
    SKIP = 6        #The type of a {@link LexerSkipAction} action.
    TYPE = 7        #The type of a {@link LexerTypeAction} action.

class LexerAction(object):

    def __init__(self, action):
        self.actionType = action
        self.isPositionDependent = False

    def __hash__(self):
        return hash(self.actionType)

    def __eq__(self, other):
        return self is other

    def __str__(self):
        return unicode(self)

    def __unicode__(self):
        return unicode(super(LexerAction, self))


#
# Implements the {@code skip} lexer action by calling {@link Lexer#skip}.
#
# <p>The {@code skip} command does not have any parameters, so this action is
# implemented as a singleton instance exposed by {@link #INSTANCE}.</p>
class LexerSkipAction(LexerAction ):

    # Provides a singleton instance of this parameterless lexer action.
    INSTANCE = None

    def __init__(self):
        super(LexerSkipAction, self).__init__(LexerActionType.SKIP)

    def execute(self, lexer):
        lexer.skip()

    def __unicode__(self):
        return u"skip"

LexerSkipAction.INSTANCE = LexerSkipAction()

#  Implements the {@code type} lexer action by calling {@link Lexer#setType}
# with the assigned type.
class LexerTypeAction(LexerAction):

    def __init__(self, type):
        super(LexerTypeAction, self).__init__(LexerActionType.TYPE)
        self.type = type

    def execute(self, lexer):
        lexer.type = self.type

    def __hash__(self):
        return hash((self.actionType, self.type))

    def __eq__(self, other):
        if self is other:
            return True
        elif not isinstance(other, LexerTypeAction):
            return False
        else:
            return self.type == other.type

    def __unicode__(self):
        return u"type(" + unicode(self.type) + u")"


# Implements the {@code pushMode} lexer action by calling
# {@link Lexer#pushMode} with the assigned mode.
class LexerPushModeAction(LexerAction):

    def __init__(self, mode):
        super(LexerPushModeAction, self).__init__(LexerActionType.PUSH_MODE)
        self.mode = mode

    # <p>This action is implemented by calling {@link Lexer#pushMode} with the
    # value provided by {@link #getMode}.</p>
    def execute(self, lexer):
        lexer.pushMode(self.mode)

    def __hash__(self):
        return hash((self.actionType, self.mode))

    def __eq__(self, other):
        if self is other:
            return True
        elif not isinstance(other, LexerPushModeAction):
            return False
        else:
            return self.mode == other.mode

    def __unicode__(self):
        return u"pushMode(" + unicode(self.mode) + u")"


# Implements the {@code popMode} lexer action by calling {@link Lexer#popMode}.
#
# <p>The {@code popMode} command does not have any parameters, so this action is
# implemented as a singleton instance exposed by {@link #INSTANCE}.</p>
class LexerPopModeAction(LexerAction):

    INSTANCE = None

    def __init__(self):
        super(LexerPopModeAction, self).__init__(LexerActionType.POP_MODE)

    # <p>This action is implemented by calling {@link Lexer#popMode}.</p>
    def execute(self, lexer):
        lexer.popMode()

    def __unicode__(self):
        return "popMode"

LexerPopModeAction.INSTANCE = LexerPopModeAction()

# Implements the {@code more} lexer action by calling {@link Lexer#more}.
#
# <p>The {@code more} command does not have any parameters, so this action is
# implemented as a singleton instance exposed by {@link #INSTANCE}.</p>
class LexerMoreAction(LexerAction):

    INSTANCE = None

    def __init__(self):
        super(LexerMoreAction, self).__init__(LexerActionType.MORE)

    # <p>This action is implemented by calling {@link Lexer#popMode}.</p>
    def execute(self, lexer):
        lexer.more()

    def __unicode__(self):
        return "more"

LexerMoreAction.INSTANCE = LexerMoreAction()

# Implements the {@code mode} lexer action by calling {@link Lexer#mode} with
# the assigned mode.
class LexerModeAction(LexerAction):

    def __init__(self, mode):
        super(LexerModeAction, self).__init__(LexerActionType.MODE)
        self.mode = mode

    # <p>This action is implemented by calling {@link Lexer#mode} with the
    # value provided by {@link #getMode}.</p>
    def execute(self, lexer):
        lexer.mode(self.mode)

    def __hash__(self):
        return hash((self.actionType, self.mode))

    def __eq__(self, other):
        if self is other:
            return True
        elif not isinstance(other, LexerModeAction):
            return False
        else:
            return self.mode == other.mode

    def __unicode__(self):
        return u"mode(" + unicode(self.mode) + u")"

# Executes a custom lexer action by calling {@link Recognizer#action} with the
# rule and action indexes assigned to the custom action. The implementation of
# a custom action is added to the generated code for the lexer in an override
# of {@link Recognizer#action} when the grammar is compiled.
#
# <p>This class may represent embedded actions created with the <code>{...}</code>
# syntax in ANTLR 4, as well as actions created for lexer commands where the
# command argument could not be evaluated when the grammar was compiled.</p>

class LexerCustomAction(LexerAction):

    # Constructs a custom lexer action with the specified rule and action
    # indexes.
    #
    # @param ruleIndex The rule index to use for calls to
    # {@link Recognizer#action}.
    # @param actionIndex The action index to use for calls to
    # {@link Recognizer#action}.
    #/
    def __init__(self, ruleIndex, actionIndex):
        super(LexerCustomAction, self).__init__(LexerActionType.CUSTOM)
        self.ruleIndex = ruleIndex
        self.actionIndex = actionIndex
        self.isPositionDependent = True

    # <p>Custom actions are implemented by calling {@link Lexer#action} with the
    # appropriate rule and action indexes.</p>
    def execute(self, lexer):
        lexer.action(None, self.ruleIndex, self.actionIndex)

    def __hash__(self):
        return hash((self.actionType, self.ruleIndex, self.actionIndex))

    def __eq__(self, other):
        if self is other:
            return True
        elif not isinstance(other, LexerCustomAction):
            return False
        else:
            return self.ruleIndex == other.ruleIndex and self.actionIndex == other.actionIndex

# Implements the {@code channel} lexer action by calling
# {@link Lexer#setChannel} with the assigned channel.
class LexerChannelAction(LexerAction):

    # Constructs a new {@code channel} action with the specified channel value.
    # @param channel The channel value to pass to {@link Lexer#setChannel}.
    def __init__(self, channel):
        super(LexerChannelAction, self).__init__(LexerActionType.CHANNEL)
        self.channel = channel

    # <p>This action is implemented by calling {@link Lexer#setChannel} with the
    # value provided by {@link #getChannel}.</p>
    def execute(self, lexer):
        lexer._channel = self.channel

    def __hash__(self):
        return hash((self.actionType, self.channel))

    def __eq__(self, other):
        if self is other:
            return True
        elif not isinstance(other, LexerChannelAction):
            return False
        else:
            return self.channel == other.channel

    def __unicode__(self):
        return u"channel(" + unicode(self.channel) + u")"

# This implementation of {@link LexerAction} is used for tracking input offsets
# for position-dependent actions within a {@link LexerActionExecutor}.
#
# <p>This action is not serialized as part of the ATN, and is only required for
# position-dependent lexer actions which appear at a location other than the
# end of a rule. For more information about DFA optimizations employed for
# lexer actions, see {@link LexerActionExecutor#append} and
# {@link LexerActionExecutor#fixOffsetBeforeMatch}.</p>
class LexerIndexedCustomAction(LexerAction):

    # Constructs a new indexed custom action by associating a character offset
    # with a {@link LexerAction}.
    #
    # <p>Note: This class is only required for lexer actions for which
    # {@link LexerAction#isPositionDependent} returns {@code true}.</p>
    #
    # @param offset The offset into the input {@link CharStream}, relative to
    # the token start index, at which the specified lexer action should be
    # executed.
    # @param action The lexer action to execute at a particular offset in the
    # input {@link CharStream}.
    def __init__(self, offset, action):
        super(LexerIndexedCustomAction, self).__init__(action.actionType)
        self.offset = offset
        self.action = action
        self.isPositionDependent = True

    # <p>This method calls {@link #execute} on the result of {@link #getAction}
    # using the provided {@code lexer}.</p>
    def execute(self, lexer):
        # assume the input stream position was properly set by the calling code
        self.action.execute(lexer)

    def __hash__(self):
        return hash((self.actionType, self.offset, self.action))

    def __eq__(self, other):
        if self is other:
            return True
        elif not isinstance(other, LexerIndexedCustomAction):
            return False
        else:
            return self.offset == other.offset and self.action == other.action



from QBCodeSpecs.antlr4.Token import Token
from QBCodeSpecs.antlr4.InputStream import InputStream
from QBCodeSpecs.antlr4.FileStream import FileStream
from QBCodeSpecs.antlr4.StdinStream import StdinStream
from QBCodeSpecs.antlr4.BufferedTokenStream import TokenStream
from QBCodeSpecs.antlr4.CommonTokenStream import CommonTokenStream
from QBCodeSpecs.antlr4.Lexer import Lexer
from QBCodeSpecs.antlr4.Parser import Parser
from QBCodeSpecs.antlr4.dfa.DFA import DFA
from QBCodeSpecs.antlr4.atn.ATN import ATN
from QBCodeSpecs.antlr4.atn.ATNDeserializer import ATNDeserializer
from QBCodeSpecs.antlr4.atn.LexerATNSimulator import LexerATNSimulator
from QBCodeSpecs.antlr4.atn.ParserATNSimulator import ParserATNSimulator
from QBCodeSpecs.antlr4.atn.PredictionMode import PredictionMode
from QBCodeSpecs.antlr4.PredictionContext import PredictionContextCache
from QBCodeSpecs.antlr4.ParserRuleContext import ParserRuleContext
from QBCodeSpecs.antlr4.tree.Tree import ParseTreeListener, ParseTreeVisitor, ParseTreeWalker, TerminalNode, ErrorNode, RuleNode
from QBCodeSpecs.antlr4.error.Errors import RecognitionException, IllegalStateException, NoViableAltException
from QBCodeSpecs.antlr4.error.ErrorStrategy import BailErrorStrategy
from QBCodeSpecs.antlr4.error.DiagnosticErrorListener import DiagnosticErrorListener
from QBCodeSpecs.antlr4.Utils import str_list



#
# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.
#

#
#  This is an InputStream that is loaded from a file all at once
#  when you construct the object.
#

import codecs
import unittest
from QBCodeSpecs.antlr4.InputStream import InputStream


class FileStream(InputStream):
    
    def __init__(self, fileName, encoding='ascii', errors='strict'):
        self.fileName = fileName
        # read binary to avoid line ending conversion
        with open(fileName, 'rb') as file:
            bytes = file.read()
            data = codecs.decode(bytes, encoding, errors)
            super(type(self), self).__init__(data)


class TestFileStream(unittest.TestCase):

    def testStream(self):
        stream = FileStream("FileStream.py")
        self.assertTrue(stream.size>0)



# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.
#/

# A lexer is recognizer that draws input symbols from a character stream.
#  lexer grammars result in a subclass of self object. A Lexer object
#  uses simplified match() and error recovery mechanisms in the interest
#  of speed.
#/
from __future__ import print_function
from io import StringIO
import sys
from QBCodeSpecs.antlr4.CommonTokenFactory import CommonTokenFactory
from QBCodeSpecs.antlr4.Recognizer import Recognizer
from QBCodeSpecs.antlr4.Token import Token
from QBCodeSpecs.antlr4.error.Errors import IllegalStateException, LexerNoViableAltException

class TokenSource(object):

    pass


class Lexer(Recognizer, TokenSource):

    DEFAULT_MODE = 0
    MORE = -2
    SKIP = -3

    DEFAULT_TOKEN_CHANNEL = Token.DEFAULT_CHANNEL
    HIDDEN = Token.HIDDEN_CHANNEL
    MIN_CHAR_VALUE = 0x0000
    MAX_CHAR_VALUE = 0x10FFFF

    def __init__(self, input, output=sys.stdout):
        super(Lexer, self).__init__()
        self._input = input
        self._output = output
        self._factory = CommonTokenFactory.DEFAULT
        self._tokenFactorySourcePair = (self, input)

        self._interp = None # child classes must populate this

        # The goal of all lexer rules/methods is to create a token object.
        #  self is an instance variable as multiple rules may collaborate to
        #  create a single token.  nextToken will return self object after
        #  matching lexer rule(s).  If you subclass to allow multiple token
        #  emissions, then set self to the last token to be matched or
        #  something nonnull so that the auto token emit mechanism will not
        #  emit another token.
        self._token = None

        # What character index in the stream did the current token start at?
        #  Needed, for example, to get the text for current token.  Set at
        #  the start of nextToken.
        self._tokenStartCharIndex = -1

        # The line on which the first character of the token resides#/
        self._tokenStartLine = -1

        # The character position of first character within the line#/
        self._tokenStartColumn = -1

        # Once we see EOF on char stream, next token will be EOF.
        #  If you have DONE : EOF ; then you see DONE EOF.
        self._hitEOF = False

        # The channel number for the current token#/
        self._channel = Token.DEFAULT_CHANNEL

        # The token type for the current token#/
        self._type = Token.INVALID_TYPE

        self._modeStack = []
        self._mode = self.DEFAULT_MODE

        # You can set the text for the current token to override what is in
        #  the input char buffer.  Use setText() or can set self instance var.
        #/
        self._text = None


    def reset(self):
        # wack Lexer state variables
        if self._input is not None:
            self._input.seek(0) # rewind the input
        self._token = None
        self._type = Token.INVALID_TYPE
        self._channel = Token.DEFAULT_CHANNEL
        self._tokenStartCharIndex = -1
        self._tokenStartColumn = -1
        self._tokenStartLine = -1
        self._text = None

        self._hitEOF = False
        self._mode = Lexer.DEFAULT_MODE
        self._modeStack = []

        self._interp.reset()

    # Return a token from self source; i.e., match a token on the char
    #  stream.
    def nextToken(self):
        if self._input is None:
            raise IllegalStateException("nextToken requires a non-null input stream.")

        # Mark start location in char stream so unbuffered streams are
        # guaranteed at least have text of current token
        tokenStartMarker = self._input.mark()
        try:
            while True:
                if self._hitEOF:
                    self.emitEOF()
                    return self._token
                self._token = None
                self._channel = Token.DEFAULT_CHANNEL
                self._tokenStartCharIndex = self._input.index
                self._tokenStartColumn = self._interp.column
                self._tokenStartLine = self._interp.line
                self._text = None
                continueOuter = False
                while True:
                    self._type = Token.INVALID_TYPE
                    ttype = self.SKIP
                    try:
                   
                        ttype = self._interp.match(self._input, self._mode)
                        # print(ttype);
                    except LexerNoViableAltException as e:
                        self.notifyListeners(e)		# report error
                        self.recover(e)
                    if self._input.LA(1)==Token.EOF:
                        self._hitEOF = True
                    if self._type == Token.INVALID_TYPE:
                        self._type = ttype
                    if self._type == self.SKIP:
                        continueOuter = True
                        break
                    if self._type!=self.MORE:
                        break
                if continueOuter:
                    continue
                if self._token is None:
                    self.emit()
                return self._token
        finally:
            # make sure we release marker after match or
            # unbuffered char stream will keep buffering
            self._input.release(tokenStartMarker)

    # Instruct the lexer to skip creating a token for current lexer rule
    #  and look for another token.  nextToken() knows to keep looking when
    #  a lexer rule finishes with token set to SKIP_TOKEN.  Recall that
    #  if token==null at end of any token rule, it creates one for you
    #  and emits it.
    #/
    def skip(self):
        self._type = self.SKIP

    def more(self):
        self._type = self.MORE

    def mode(self, m):
        self._mode = m

    def pushMode(self, m):
        if self._interp.debug:
            print("pushMode " + str(m), file=self._output)
        self._modeStack.append(self._mode)
        self.mode(m)

    def popMode(self):
        if len(self._modeStack)==0:
            raise Exception("Empty Stack")
        if self._interp.debug:
            print("popMode back to "+ self._modeStack[:-1], file=self._output)
        self.mode( self._modeStack.pop() )
        return self._mode

    # Set the char stream and reset the lexer#/
    @property
    def inputStream(self):
        return self._input

    @inputStream.setter
    def inputStream(self, input):
        self._input = None
        self._tokenFactorySourcePair = (self, self._input)
        self.reset()
        self._input = input
        self._tokenFactorySourcePair = (self, self._input)

    @property
    def sourceName(self):
        return self._input.sourceName

    # By default does not support multiple emits per nextToken invocation
    #  for efficiency reasons.  Subclass and override self method, nextToken,
    #  and getToken (to push tokens into a list and pull from that list
    #  rather than a single variable as self implementation does).
    #/
    def emitToken(self, token):
        self._token = token

    # The standard method called to automatically emit a token at the
    #  outermost lexical rule.  The token object should point into the
    #  char buffer start..stop.  If there is a text override in 'text',
    #  use that to set the token's text.  Override self method to emit
    #  custom Token objects or provide a new factory.
    #/
    def emit(self):
        t = self._factory.create(self._tokenFactorySourcePair, self._type, self._text, self._channel, self._tokenStartCharIndex,
                                 self.getCharIndex()-1, self._tokenStartLine, self._tokenStartColumn)
        self.emitToken(t)
        return t

    def emitEOF(self):
        cpos = self.column
        lpos = self.line
        eof = self._factory.create(self._tokenFactorySourcePair, Token.EOF, None, Token.DEFAULT_CHANNEL, self._input.index,
                                   self._input.index-1, lpos, cpos)
        self.emitToken(eof)
        return eof

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, type):
        self._type = type

    @property
    def line(self):
        return self._interp.line

    @line.setter
    def line(self, line):
        self._interp.line = line

    @property
    def column(self):
        return self._interp.column

    @column.setter
    def column(self, column):
        self._interp.column = column

    # What is the index of the current character of lookahead?#/
    def getCharIndex(self):
        return self._input.index

    # Return the text matched so far for the current token or any
    #  text override.
    @property
    def text(self):
        if self._text is not None:
            return self._text
        else:
            return self._interp.getText(self._input)

    # Set the complete text of self token; it wipes any previous
    #  changes to the text.
    @text.setter
    def text(self, txt):
        self._text = txt

    # Return a list of all Token objects in input char stream.
    #  Forces load of all tokens. Does not include EOF token.
    #/
    def getAllTokens(self):
        tokens = []
        t = self.nextToken()
        while t.type!=Token.EOF:
            tokens.append(t)
            t = self.nextToken()
            # print(t.type);
        return tokens

    def notifyListeners(self, e):
        start = self._tokenStartCharIndex
        stop = self._input.index
        text = self._input.getText(start, stop)
        msg = u"token recognition error at: '" + self.getErrorDisplay(text) + u"'"
        listener = self.getErrorListenerDispatch()
        listener.syntaxError(self, None, self._tokenStartLine, self._tokenStartColumn, msg, e)

    def getErrorDisplay(self, s):
        with StringIO() as buf:
            for c in s:
                buf.write(unicode(self.getErrorDisplayForChar(c)))
            return buf.getvalue()

    def getErrorDisplayForChar(self, c):
        if ord(c[0])==Token.EOF:
            return "<EOF>"
        elif c==u'\n':
            return u"\\n"
        elif c==u'\t':
            return u"\\t"
        elif c==u'\r':
            return u"\\r"
        else:
            return c

    def getCharErrorDisplay(self, c):
        return u"'" + self.getErrorDisplayForChar(c) + u"'"

    # Lexers can normally match any char in it's vocabulary after matching
    #  a token, so do the easy thing and just kill a character and hope
    #  it all works out.  You can instead use the rule invocation stack
    #  to do sophisticated error recovery if you are in a fragment rule.
    #/
    def recover(self, re):
        if self._input.LA(1) != Token.EOF:
            if isinstance(re, LexerNoViableAltException):
                    # skip a char and try again
                    self._interp.consume(self._input)
            else:
                # TODO: Do we lose character or line position information?
                self._input.consume()




#
# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.
#


# A set of utility routines useful for all kinds of ANTLR trees.#
from io import StringIO
import QBCodeSpecs.antlr4
from QBCodeSpecs.antlr4.atn.ATN import ATN
from QBCodeSpecs.antlr4.Token import Token
from QBCodeSpecs.antlr4.Utils import escapeWhitespace
from QBCodeSpecs.antlr4.tree.Tree import RuleNode, ErrorNode, TerminalNode

class Trees(object):

    # Print out a whole tree in LISP form. {@link #getNodeText} is used on the
    #  node payloads to get the text for the nodes.  Detect
    #  parse trees and extract data appropriately.
    @classmethod
    def toStringTree(cls, t, ruleNames=None, recog=None, index = 0):
        if recog is not None:
            ruleNames = recog.ruleNames
        s = escapeWhitespace(cls.getNodeText(t, ruleNames), False)
        if t.getChildCount()==0:
            return s
        with StringIO() as buf:
            buf.write(u"\n")

            tab_num = index
            while tab_num > 0:
                buf.write(u"    ")
                tab_num = tab_num - 1

            buf.write(u"(")
            buf.write(s)
            buf.write(u' ')
            for i in range(0, t.getChildCount()):
                if i > 0:
                    buf.write(u' ')
                next_index = index + 1
                buf.write(cls.toStringTree(t.getChild(i), ruleNames, recog=None, index=next_index))
            buf.write(u")")
            return buf.getvalue()

    @classmethod
    def getNodeText(cls, t, ruleNames=None, recog=None):
        if recog is not None:
            ruleNames = recog.ruleNames
        if ruleNames is not None:
            if isinstance(t, RuleNode):
                if t.getAltNumber()!=ATN.INVALID_ALT_NUMBER:
                    return ruleNames[t.getRuleIndex()]+":"+str(t.getAltNumber())
                return ruleNames[t.getRuleIndex()]
            elif isinstance( t, ErrorNode):
                return unicode(t)
            elif isinstance(t, TerminalNode):
                if t.symbol is not None:
                    return t.symbol.text
        # no recog for rule names
        payload = t.getPayload()
        if isinstance(payload, Token ):
            return payload.text
        return unicode(t.getPayload())


    # Return ordered list of all children of this node
    @classmethod
    def getChildren(cls, t):
        return [ t.getChild(i) for i in range(0, t.getChildCount()) ]

    # Return a list of all ancestors of this node.  The first node of
    #  list is the root and the last is the parent of this node.
    #
    @classmethod
    def getAncestors(cls, t):
        ancestors = []
        t = t.getParent()
        while t is not None:
            ancestors.append(0, t) # insert at start
            t = t.getParent()
        return ancestors

    @classmethod
    def findAllTokenNodes(cls, t, ttype):
        return cls.findAllNodes(t, ttype, True)

    @classmethod
    def findAllRuleNodes(cls, t, ruleIndex):
        return cls.findAllNodes(t, ruleIndex, False)

    @classmethod
    def findAllNodes(cls, t, index, findTokens):
        nodes = []
        cls._findAllNodes(t, index, findTokens, nodes)
        return nodes

    @classmethod
    def _findAllNodes(cls, t, index, findTokens, nodes):
        from QBCodeSpecs.antlr4.ParserRuleContext import ParserRuleContext
        # check this node (the root) first
        if findTokens and isinstance(t, TerminalNode):
            if t.symbol.type==index:
                nodes.append(t)
        elif not findTokens and isinstance(t, ParserRuleContext):
            if t.ruleIndex == index:
                nodes.append(t)
        # check children
        for i in range(0, t.getChildCount()):
            cls._findAllNodes(t.getChild(i), index, findTokens, nodes)

    @classmethod
    def descendants(cls, t):
        nodes = [t]
        for i in range(0, t.getChildCount()):
            nodes.extend(cls.descendants(t.getChild(i)))
        return nodes



#!/usr/bin/env python
# -coding:utf-8
import sys
from QBCodeSpecs.objcRecoginition.ObjectiveCParserListener import *
from QBCodeSpecs.driver.ViolationDataMananger import *
from QBCodeSpecs.driver.Violation import *
import re
import linecache


class WeakDelegateRuleListener(ObjectiveCParserListener):

	def __init__(self, violation):
		super(WeakDelegateRuleListener,self).__init__()
		self.violation = violation
		self.startline = 0
		self.startcolumn = 0


	def send_violation(self, startline, startcolumn):

		path = self.violation.path
		configInfo = self.violation.configinfo

		violInfo = Violation(path, configInfo)
		violInfo.startline = startline
		violInfo.startcolumn = startcolumn

		# violInfo.set_rulename("")
		violInfo.set_priority("1")
		violInfo.set_rulecategory("Delegate属性定义")
		codeStr = linecache.getline(path,self.startline)
		# print "source code:" + codeStr
		violInfo.message = "delegate的属性使用retain来修饰可能存在循环引用"
		#"Conforming to regular expressions:" + "\"" + self.regex + "\"" 

		democode = "@property (nonatomic, weak) id&lt;ObjcDelegate&gt; delegate;"
		violInfo.set_democode(democode)
		noncompliant = "@property (nonatomic, retain) id&lt;ObjcDelegate&gt; delegate;"
		violInfo.set_noncompliant_code(noncompliant)
		violationDataMananger.addViolation(violInfo)

	def exitPropertyDeclaration(self, ctx):
		hasRetainAttribute = False
		# 包含<Delegate>
		hasDelegateFlag = False
		# hasDelegateDeclarator = False
		startline = 0
		startcolumn = 0

		# 属性
		attributesList = ctx.propertyAttributesList().propertyAttribute()
		for x in attributesList:
			# print "attribute:--" + x.getText()
			if "retain" in x.getText():
				hasRetainAttribute = True

		# 类型
		declaration = ctx.fieldDeclaration()
		qualifierList = declaration.specifierQualifierList().typeSpecifier()
		for x in qualifierList:
			# print "typeSpecifier:--" + x.getText()
			qualifierStr = x.getText()
			typeStr = re.findall(r"<(.+?)>", qualifierStr)
			for y in typeStr:
				# print "re--"+y
				if "delegate" in y or "Delegate" in y:
					# print "in hasDelegateFlag"
					token = x.start
					startline = token.line
					startcolumn = token.column
					hasDelegateFlag = True
			
			

		# 变量
		# declaratorList = declaration.fieldDeclaratorList().fieldDeclarator()
		# for x in declaratorList:
		# 	print "fieldDeclarator:--" + x.getText()
		# 	if "delegate" in x.getText() or "Delegate" in x.getText():
		# 		hasDelegateDeclarator = True

		if hasRetainAttribute and hasDelegateFlag:
			self.send_violation(startline, startcolumn)
			# print "retain delegate"



